package com.unistrong.pogoserialtester;

import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.lovdream.LovUtil;
import com.unistrong.pogoservicelib.pogoservice.pin.Gpio1;
import com.unistrong.pogoservicelib.pogoservice.pin.Gpio2;
import com.unistrong.pogoservicelib.pogoservice.pin.Irq;
import com.unistrong.pogoservicelib.pogoservice.pin.VbatPower;
import com.unistrong.pogoservicelib.pogoservice.pin.VbusPower;
import com.unistrong.pogoservicelib.pogoservice.serial.RealSerial;
import com.unistrong.pogoservicelib.pogoservice.serial.template.IDataReceiveCallback;
import com.unistrong.pogoservicelib.pogoservice.status.PinStatus;

import java.util.Timer;
import java.util.TimerTask;

import androidx.appcompat.app.AppCompatActivity;

public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MainActivity";
    private static final boolean DEBUG = BuildConfig.DEBUG;
    private Switch mPowerSwitch_three,mPowerSwitch_five;
    private Switch mPinSwitch_one,mPinSwitch_two;
    private Button mReadIrqButton;
    private Switch mSerialSwitcher;
    private TextView mSerialRecvResult;
    private SendSerialThread sendThread;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mPowerSwitch_three = findViewById(R.id.sw_power_three);
        mPowerSwitch_five = findViewById(R.id.sw_power_five);


        mPinSwitch_one = findViewById(R.id.sw_pin_1);
        mPinSwitch_two = findViewById(R.id.sw_pin_2);

        mReadIrqButton = findViewById(R.id.bt_read_irq);
        mSerialRecvResult = findViewById(R.id.tv_result);
        mSerialSwitcher = findViewById(R.id.sw_serial);

        mPowerSwitch_three.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    VbatPower.getInstance().setELevel(PinStatus.HIGH);
                } else {
                    VbatPower.getInstance().setELevel(PinStatus.LOW);
                    mSerialSwitcher.setChecked(false);
                    mPinSwitch_one.setChecked(false);
                    mPinSwitch_two.setChecked(false);
                }
            }
        });
        mPowerSwitch_five.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    VbusPower.getInstance().setELevel(PinStatus.HIGH);
                } else {
                    VbusPower.getInstance().setELevel(PinStatus.LOW);
                    mSerialSwitcher.setChecked(false);
                    mPinSwitch_one.setChecked(false);
                    mPinSwitch_two.setChecked(false);
                }
            }
        });


        mPinSwitch_one.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    Log.d(TAG, "vbat = " + VbatPower.getInstance().getELevel());
                    Log.d(TAG, "vbus = " + VbusPower.getInstance().getELevel());
                    if ((!LovUtil.gUseBackNode)
                            && (VbatPower.getInstance().getELevel() == PinStatus.LOW
                            || VbusPower.getInstance().getELevel() == PinStatus.LOW)) {
                        Toast.makeText(getApplicationContext(), R.string.please_open_power, Toast.LENGTH_SHORT).show();
                        mPinSwitch_one.setChecked(false);
                        return;
                    }
                    Gpio1.getInstance().setELevel(PinStatus.HIGH);
                } else {
                    Gpio1.getInstance().setELevel(PinStatus.LOW);
                }
            }
        });


        mPinSwitch_two.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    Log.d(TAG, "vbat = " + VbatPower.getInstance().getELevel());
                    Log.d(TAG, "vbus = " + VbusPower.getInstance().getELevel());
                    if ((!LovUtil.gUseBackNode)
                            && (VbatPower.getInstance().getELevel() == PinStatus.LOW
                            || VbusPower.getInstance().getELevel() == PinStatus.LOW)) {
                        Toast.makeText(getApplicationContext(), R.string.please_open_power, Toast.LENGTH_SHORT).show();
                        mPinSwitch_two.setChecked(false);
                        return;
                    }
                    Gpio2.getInstance().setELevel(PinStatus.HIGH);
                } else {
                    Gpio2.getInstance().setELevel(PinStatus.LOW);
                }
            }
        });

        mReadIrqButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                PinStatus result = Irq.getInstance().getELevel();

                Toast.makeText(getApplicationContext(),
                        String.valueOf(result == PinStatus.LOW ? 0 : 1),
                        Toast.LENGTH_SHORT).show();
            }
        });


        mSerialSwitcher.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    RealSerial.getInstance().registerReceiveCallback(new IDataReceiveCallback() {
                        @Override
                        public void onDataReceived(byte[] data, int len) {
                            final String str = new String(data).substring(0, len);
                            runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    if (mSerialRecvResult.getText().length() > 1024) {
                                        mSerialRecvResult.setText(str);
                                    } else {
                                        mSerialRecvResult.append(str);
                                    }
                                }
                            });
                        }
                    });
                    if(RealSerial.getInstance().open(115200)) {
                        sendThread = new SendSerialThread();
                        sendThread.start();
                    } else {
                        mSerialSwitcher.setChecked(false);
                    }
                } else {
                    mSerialRecvResult.setText("");
                    sendThread.cancel();
                    sendThread = null;
                    RealSerial.getInstance().unregisterReceiveCallback();
                    RealSerial.getInstance().close();
                }
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();

    }

    @Override
    protected void onPause() {
        super.onPause();
        VbatPower.getInstance().setELevel(PinStatus.LOW);
        VbusPower.getInstance().setELevel(PinStatus.LOW);
        Gpio1.getInstance().setELevel(PinStatus.LOW);
        Gpio2.getInstance().setELevel(PinStatus.LOW);
        mSerialRecvResult.setText("");
        if(sendThread !=null) {
            sendThread.cancel();
            sendThread = null;
        }
        RealSerial.getInstance().unregisterReceiveCallback();
        RealSerial.getInstance().close();

        mPowerSwitch_three.setChecked(false);
        mPowerSwitch_five.setChecked(false);
        mPinSwitch_two.setChecked(false);
        mPinSwitch_one.setChecked(false);
        mSerialSwitcher.setOnCheckedChangeListener(null);

    }

    private class SendSerialThread extends Thread {
        private int data;
        private Timer timer;
        private TimerTask task = new TimerTask() {
            @Override
            public void run() {
                String str = String.valueOf(data);
                RealSerial.getInstance().sendData(str.getBytes(), str.length());
                if (DEBUG) Log.d(TAG, "Send data: " + str);
                RealSerial.getInstance().readDataNoWait();
                data++;
            }
        };

        public void start() {
            data = 0;
            if (null != timer) {
                timer.cancel();
            }
            timer = new Timer();
            timer.schedule(task, 100, 1000);
            if (DEBUG) Log.d(TAG, "Start test.");
        }

        private void cancel() {
            timer.cancel();
            timer = null;
            interrupt();
            if (DEBUG) Log.d(TAG, "Stop test.");
        }
    }
}
