package com.unistrong.miscutils;

import android.util.Log;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

public class FileUtil {
    private static final String TAG = "FileUtil";
    private static final boolean DEBUG = false;

    private String mNodeName;
    private BufferedWriter mBufferedWriter = null;
    private BufferedReader mBufferedReader = null;
    private boolean mIsWriterPrepared = false;
    private boolean mIsReaderPrepared = false;
    private boolean mIsReadOnly = false;

    public FileUtil(String nodeName) {
        this.mNodeName = nodeName;
    }

    public synchronized boolean open(boolean readonly) {
        mIsReadOnly = readonly;
        if (DEBUG) {
            Log.i(TAG, "open(" + readonly + ")");
        }
        if (!mIsReadOnly) {
            try {
                mBufferedWriter = new BufferedWriter(new FileWriter(mNodeName));
                mIsWriterPrepared = true;
            } catch (IOException e) {
                mIsWriterPrepared = false;
                Log.e(TAG, "open(), Can`t open write buffer");
            }
            if (DEBUG) {
                Log.i(TAG, "open(), Write buffer prepared? " + mIsWriterPrepared);
            }
        }

        try {
            mBufferedReader = new BufferedReader(new FileReader(mNodeName));
            mIsReaderPrepared = true;
        } catch (IOException e) {
            mIsReaderPrepared = false;
            Log.e(TAG, "open(), Can`t open reader buffer");
        }
        if (DEBUG) {
            Log.d(TAG, "open(), Reader buffer prepared? " + mIsReaderPrepared);
        }

        if (DEBUG) {
            Log.d(TAG, "open(), End");
        }
        return isIsPrepared();
    }

    public synchronized void close() {
        if (DEBUG) {
            Log.d(TAG, "close(), Begin");
        }
        if (!mIsReadOnly) {
            try {
                mBufferedWriter.close();
            } catch (NullPointerException npe) {
                //ignore
                Log.e(TAG, "close(), Write buffer is null!");
            } catch (IOException e) {
                //ignore
                Log.e(TAG, "close(), Write buffer close failed!");
            } finally {
                mBufferedWriter = null;
                mIsWriterPrepared = false;
                if (DEBUG) {
                    Log.d(TAG, "mBufferedWriter.close()");
                }
            }
        }
        try {
            if (null != mBufferedReader) {
                mBufferedReader.close();
            }
            if (DEBUG) {
                Log.d(TAG, "mBufferedReader.close()");
            }
        } catch (NullPointerException npe) {
            //ignore
            Log.e(TAG, "close(), Reader buffer is null!");
        } catch (IOException e) {
            //ignore
            Log.e(TAG, "close(), Reader buffer close failed!");
        } finally {
            mBufferedReader = null;
            mIsReaderPrepared = false;
        }
        if (DEBUG) {
            Log.d(TAG, "close(), End");
        }
    }

    public synchronized String readLine() throws IOException {
        String string = "";
        if (mIsReaderPrepared && null != mBufferedReader) {
            if (mBufferedReader.ready()) {
                string = mBufferedReader.readLine();
            }
        }
        if (DEBUG) {
            Log.d(TAG, "readLine(), Msg = " + string);
        }
        return string;
    }

    /**
     * Reads a single character.
     *
     * @return The character read, as an integer in the range
     * 0 to 65535 (<tt>0x00-0xffff</tt>), or -1 if the
     * end of the stream has been reached
     * @throws IOException If an I/O error occurs
     */
    public synchronized int readChar() throws IOException {
        int c = -1;
        if (mIsReaderPrepared && null != mBufferedReader) {
            if (mBufferedReader.ready()) {
                c = mBufferedReader.read();
            }
        }
        if (DEBUG) {
            Log.d(TAG, "readChar(), Char = " + String.valueOf(c));
        }
        return c;
    }

    public synchronized int readChars(char[] cbuf, int off, int len) throws IOException {
        int count = 0;
        if (mIsReaderPrepared && null != mBufferedReader) {
            if (mBufferedReader.ready()) {
                count = mBufferedReader.read(cbuf, off, len);
            }
        }
        if (DEBUG) {
            Log.d(TAG, "readChars(), Char = " + String.valueOf(cbuf) + ", len = " + count);
        }
        return count;
    }

    public synchronized void write(String msg) throws IOException {
        if (DEBUG) {
            Log.d(TAG, "write(), Buffers prepared? " + (!mIsReadOnly && mIsWriterPrepared));
            Log.d(TAG, "write(), null == mBufferedWriter? " + (null == mBufferedWriter));
        }
        if (mIsReadOnly || !mIsWriterPrepared || null == mBufferedWriter) return;
        mBufferedWriter.write(msg);
        mBufferedWriter.flush();
        if (DEBUG) {
            Log.d(TAG, "write(), Msg = " + String.valueOf(msg));
        }
    }

    public void write(Object object) throws IOException {
        write(String.valueOf(object));
    }

    public String getNodeName() {
        return mNodeName;
    }

    public BufferedWriter getBufferedWriter() {
        return mBufferedWriter;
    }

    public BufferedReader getBufferedReader() {
        return mBufferedReader;
    }

    public boolean isIsPrepared() {
        if (mIsReadOnly) {
            if (DEBUG) {
                Log.d(TAG, "isIsPrepared(), Reader buffer prepared? " + mIsReaderPrepared);
            }
            return mIsReaderPrepared;
        } else {
            if (DEBUG) {
                Log.d(TAG, "isIsPrepared(), Buffers prepared? " + (mIsWriterPrepared && mIsReaderPrepared));
            }
            return mIsWriterPrepared && mIsReaderPrepared;
        }
    }
}