package com.unistrong.pogoservicelib.pogoservice.serial;

import com.unistrong.pogoservicelib.pogoservice.devices.DeviceFactory;
import com.unistrong.pogoservicelib.pogoservice.serial.template.SerialPortImpl;
import com.unistrong.pogoservicelib.pogoservice.serial.template.SoManager;

public final class RealSerial extends SerialPortImpl {
    private RealSerial() {
        mPath = DeviceFactory.getInstance().getDevice().getPogoRealSerialPort();
    }

    public static RealSerial getInstance() {
        return REAL_SERIAL_HOLDER.BEAN;
    }

    private static final class REAL_SERIAL_HOLDER {
        private static final RealSerial BEAN = new RealSerial();
    }

    @Override
    public boolean open(int baudRate) {
        byte[] cache = new byte[1];
        boolean isSuccess = SoManager.openDevice(mPath, baudRate, 8, 0, 1);
        SoManager.readData(cache);
        return isSuccess;
    }

    @Override
    public void close() {
        SoManager.closeDevice();
    }

    @Override
    public void readData() {
        byte[] data = new byte[1024];
        int len;
        while (true) {
            len = SoManager.readData(data);
            if (len > 0 && null != mDataRecvCallback) {
                mDataRecvCallback.onDataReceived(data, len);
                break;
            } else {
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    //ignore
                }
            }
        }
    }

    public void readDataNoWait() {
        byte[] buffer = new byte[1024];
        int len = SoManager.readData(buffer);
        if (len > 0 && null != mDataRecvCallback) {
            mDataRecvCallback.onDataReceived(buffer, len);
        }
    }

    @Override
    public void sendData(byte[] data, int len) {
        byte[] msg = new byte[len];
        for (int i = 0; i < len; i++) {
            msg[i] = data[i];
        }
        SoManager.sendData(msg);
    }
}
