package com.unistrong.pogoservicelib.pogoservice.service;

import android.app.Service;
import android.content.Intent;
import android.os.Binder;
import android.os.IBinder;
import android.util.Log;

import com.unistrong.pogoservicelib.BuildConfig;
import com.unistrong.pogoservicelib.pogoservice.pin.Gpio1;
import com.unistrong.pogoservicelib.pogoservice.pin.Gpio2;
import com.unistrong.pogoservicelib.pogoservice.pin.Irq;
import com.unistrong.pogoservicelib.pogoservice.pin.VbatPower;
import com.unistrong.pogoservicelib.pogoservice.pin.VbusPower;
import com.unistrong.pogoservicelib.pogoservice.pin.template.IStatusChangeCallback;
import com.unistrong.pogoservicelib.pogoservice.status.PinStatus;
import com.unistrong.pogoservicelib.pogoservice.serial.RealSerial;
import com.unistrong.pogoservicelib.pogoservice.serial.VirtualSerial;
import com.unistrong.pogoservicelib.pogoservice.serial.template.IDataReceiveCallback;

public class PogoService extends Service {
    private static final String TAG = "PogoService";
    private static final boolean DEBUG = BuildConfig.DEBUG;

    public static final String ACTION_SERVICE_NEED_REBOOT = "pogo_service_need_reboot";

    private IStatusChangeCallback mIrqStatusChangeCallback = new IStatusChangeCallback() {
        @Override
        public boolean onIrqStatusChange(PinStatus level) {
            return false;
        }
    };

    private PogoBinder mPogoManager = new PogoBinder();

    @Override
    public void onCreate() {
        super.onCreate();
        VbusPower.getInstance().init();
        VbatPower.getInstance().init();
        Gpio1.getInstance().init();
        Gpio2.getInstance().init();
        Irq.getInstance().init();
        Irq.getInstance().registerObserver(getApplicationContext(), mIrqStatusChangeCallback);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        return START_NOT_STICKY;
    }

    @Override
    public IBinder onBind(Intent intent) {
        if (null == mPogoManager) {
            mPogoManager = new PogoBinder();
        }
        return mPogoManager;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        return super.onUnbind(intent);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        VbusPower.getInstance().deInit();
        VbatPower.getInstance().deInit();
        Gpio1.getInstance().deInit();
        Gpio2.getInstance().deInit();
        Irq.getInstance().unregisterObserver();
        Irq.getInstance().deInit();
    }

    public class PogoBinder extends Binder {
        // Power
        public void setVbus(boolean on) {
            VbusPower.getInstance().setELevel(on ? PinStatus.HIGH : PinStatus.LOW);
        }

        public boolean isVbusOn() {
            boolean isOn = PinStatus.HIGH == VbusPower.getInstance().getELevel();
            return isOn;
        }

        public void setVbat(boolean on) {
            VbatPower.getInstance().setELevel(on ? PinStatus.HIGH : PinStatus.LOW);
        }

        public boolean isVbatOn() {
            boolean isOn = PinStatus.HIGH == VbatPower.getInstance().getELevel();
            return isOn;
        }

        //GPIO
        public void setGpio1(boolean high) {
            Gpio1.getInstance().setELevel(high ? PinStatus.HIGH : PinStatus.LOW);
        }

        public boolean isGpio1High() {
            boolean isHigh = PinStatus.HIGH == Gpio1.getInstance().getELevel();
            return isHigh;
        }

        public void setGpio2(boolean high) {
            Gpio2.getInstance().setELevel(high ? PinStatus.HIGH : PinStatus.LOW);
        }

        public boolean isGpio2High() {
            boolean isHigh = PinStatus.HIGH == Gpio2.getInstance().getELevel();
            return isHigh;
        }

        public boolean isIrqLow() {
            boolean isLow = PinStatus.LOW == Irq.getInstance().getELevel();
            return isLow;
        }

        //Real serial
        public synchronized boolean openRealSerial(int baudRate, IDataReceiveCallback receiveCallback) {
            RealSerial.getInstance().registerReceiveCallback(receiveCallback);
            boolean result = RealSerial.getInstance().open(baudRate);
            Log.d(TAG, "open result: " + result);
            try {
                Thread.sleep(300);
            } catch (InterruptedException e) {
                //ignore
            }
            return result;
        }

        public synchronized void sendToRealSerial(byte[] data, int len) {
            RealSerial.getInstance().sendData(data, len);
        }

        public synchronized void readFromRealSerial() {
            RealSerial.getInstance().readData();
        }

        public synchronized void closeRealSerial() {
            RealSerial.getInstance().unregisterReceiveCallback();
            RealSerial.getInstance().close();
        }

        //Virtual serial
        public synchronized boolean openVirtualSerial(int baudRate, IDataReceiveCallback receiveCallback) {
            boolean isSuccess = false;
            VirtualSerial.getInstance().registerReceiveCallback(receiveCallback);
            isSuccess = VirtualSerial.getInstance().open(baudRate);
            try {
                Thread.sleep(300);

            } catch (InterruptedException e) {
                //ignore
            }
            return isSuccess;
        }

        public synchronized void sendToVirtualSerial(byte[] data, int len) {
            VirtualSerial.getInstance().sendData(data, len);
        }

        public synchronized void readFromVirtualSerial() {
            VirtualSerial.getInstance().readData();
        }

        public synchronized void closeVirtualSerial() {
            VirtualSerial.getInstance().unregisterReceiveCallback();
            VirtualSerial.getInstance().close();
        }
    }
}
