#include <jni.h>
#include <android/log.h>
#include <termios.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>
#include <errno.h>

#define LOG_TAG  "uartserial"

#define LOGI(fmt, args...) __android_log_print(ANDROID_LOG_INFO,  LOG_TAG, fmt, ##args)
#define LOGD(fmt, args...) __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, fmt, ##args)
#define LOGE(fmt, args...) __android_log_print(ANDROID_LOG_ERROR, LOG_TAG, fmt, ##args)


//D:\workspace\BarcodeSacn\pogoservicelib\src\main\java\com\unistrong\pogoservicelib\pogoservice\serial\template\SoManager.java
static const char *className = "com/unistrong/pogoservicelib/pogoservice/serial/template/SoManager";

int fd;

static speed_t getBaudrate(jint baudrate)
{
	switch(baudrate) {
	case 0: return B0;
	case 50: return B50;
	case 75: return B75;
	case 110: return B110;
	case 134: return B134;
	case 150: return B150;
	case 200: return B200;
	case 300: return B300;
	case 600: return B600;
	case 1200: return B1200;
	case 1800: return B1800;
	case 2400: return B2400;
	case 4800: return B4800;
	case 9600: return B9600;
	case 19200: return B19200;
	case 38400: return B38400;
	case 57600: return B57600;
	case 115200: return B115200;
	case 230400: return B230400;
	case 460800: return B460800;
	case 500000: return B500000;
	case 576000: return B576000;
	case 921600: return B921600;
	case 1000000: return B1000000;
	case 1152000: return B1152000;
	case 1500000: return B1500000;
	case 2000000: return B2000000;
	case 2500000: return B2500000;
	case 3000000: return B3000000;
	case 3500000: return B3500000;
	case 4000000: return B4000000;
	default: return 9600;
	}
}

static int set_opt(int baudrate,int bits,int event,int stop){
	struct termios cfg;
		tcflush(fd,TCIOFLUSH);
		if(tcgetattr(fd,&cfg)){
			LOGE("--JNI tcgetattr failed!");
			close(fd);
			return -1;
		}
		cfg.c_iflag &=~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
		cfg.c_oflag &= ~OPOST;
		cfg.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
		cfg.c_cflag &= ~CSIZE;
	    //数据位
		switch( bits )
	    {
		case 5:
		    cfg.c_cflag |= CS5;
			break;
		case 6:
		    cfg.c_cflag |= CS6;
			break;
	    case 7:
	        cfg.c_cflag |= CS7;
	        break;
	    case 8:
	    	cfg.c_cflag |= CS8;
	        break;
	    }
		//校验位
		 switch(event)
		 {
		    case 1:                     //奇校验
		    	cfg.c_cflag |= PARENB;
		    	cfg.c_cflag |= PARODD;
		    	cfg.c_iflag |= (INPCK | ISTRIP);
		        break;
		    case 2:                     //偶校验
		    	cfg.c_iflag |= (INPCK | ISTRIP);
		    	cfg.c_cflag |= PARENB;
		    	cfg.c_cflag &= ~PARODD;
		        break;
		    case 0:                    //无校验
		    	cfg.c_cflag &= ~PARENB;
		        break;
		 }
		 //停止位  原来的是1位停止位
		  if( stop == 1 )
		    {
			  cfg.c_cflag &=  ~CSTOPB;
		    }
		    else if ( stop == 2 )
		    {
		    	cfg.c_cflag |=  CSTOPB;
		    }

		cfg.c_cflag &= ~CRTSCTS;//no flow control
		tcsetattr(fd,TCSANOW,&cfg);
		tcflush(fd, TCIOFLUSH);
	    tcsetattr(fd, TCSANOW, &cfg);
	    tcflush(fd, TCIOFLUSH);
	    tcflush(fd, TCIOFLUSH);

	    //默认是B115200
	 	if(cfsetispeed(&cfg,getBaudrate(baudrate))){
	    		LOGE("--JNI set input baudrate failed!");
	    	    	close(fd);
	    }
	    if(cfsetospeed(&cfg,getBaudrate(baudrate))){
	    	LOGE("--JNI set output baudrate failed!");
	    	    	close(fd);
	    }
	    if(tcsetattr(fd,TCSANOW,&cfg)){
	    	LOGE("--JNI tcsetattr failed!");
	    	close(fd);
	    	return -1;
	    }
}


int write_file(char *devpath, char val){
   int fdd = -1;
   int ret = 0;

	char buf = val;
	fdd = open(devpath, O_WRONLY|O_SYNC);
	if (fdd < 0) {
		//LOGD(TAG, "write_file open error\n");
		LOGE("%s  O_WRONLY   error  ", __FUNCTION__);
		return -1;
	}
	//LOGD("write %s %s  %d ", __FUNCTION__, devpath, buf);
	ret = write(fdd, &buf, sizeof(buf));
	if (ret != sizeof(buf)) {
		//LOGD(TAG, "write_file  write error ret:%d  buf size:%d\n", ret, sizeof(buf));
		LOGE("%s  write_file: %d %d ", __FUNCTION__, ret, sizeof(buf));
		close(fdd);
		return -1;
	}
	close(fdd);

   return ret;
}


JNIEXPORT jint  GPS_OnOff(JNIEnv* env,jobject thiz, jint onoff){
//	LOGD("--JNI  GPS_OnOff()   onoff=%d",onoff);
	int ret=0;
	if(onoff>0){
		ret=write_file("/sys/class/bd_gps/bd_gps/bd_enable",'1');
	}else
		ret=write_file("/sys/class/bd_gps/bd_gps/bd_enable",'0');
	return ret;
}

JNIEXPORT jint  LaGaoOrLadi(JNIEnv* env,jobject thiz, jstring path,jint onoff){
//	LOGD("--JNI LaGaoOrLadi  onoff=%d",onoff);
	char* str_path=(char*)(*env)->GetStringUTFChars(env,path,JNI_FALSE);

	int ret=0;
	if(onoff>0){
		ret=write_file(str_path,'1');
	}else
		ret=write_file(str_path,'0');
	(*env)->ReleaseStringUTFChars(env,path,str_path);
	return ret;
}



JNIEXPORT jint JNI_OpenDev(JNIEnv* env,jobject thiz,jstring path,jint baudrate,jint bits, jint event, jint stop){
//	LOGD("--JNI  JNI_OpenDev() \n");
	const char* chr_path=(const char*)(*env)->GetStringUTFChars(env,path,JNI_FALSE);

//	LOGD("--JNI  JNI_OpenDev() : %s \n",chr_path);
	fd=open(chr_path,O_RDWR|O_NOCTTY|0);
	set_opt(baudrate,bits,event,stop);

    (*env)->ReleaseStringUTFChars(env,path,chr_path);

    return fd;
}

JNIEXPORT jint JNI_CloseDev(JNIEnv* env,jobject thiz,jint fdevice){
	close(fdevice);
	return 0;
}

JNIEXPORT jint JNI_Write(JNIEnv* env,jobject thiz,jbyteArray buffer,jint size,jint fdevice){

    //	LOGD("--JNI  JNI_GnssWrite() \n");
	unsigned char *char_buf=(unsigned char*)(*env)->GetByteArrayElements(env,buffer,NULL);
	int lenght=write(fdevice,char_buf,size);
	(*env)->ReleaseByteArrayElements(env,buffer,char_buf,0);
	return lenght;
}


JNIEXPORT jint JNICALL JNI_MinReadDev(JNIEnv *env,jobject obj, jbyteArray buffer, jint size,jint fdevice) {

	unsigned char *buf_char = (unsigned char*) ((*env)->GetByteArrayElements(env, buffer,
			NULL));
	int lenth = 0;
	int count;
	int ret = 0;
	fd_set rd;
	struct timeval tv = { 0, 50*1000 };
	memset(buf_char,0,size);//M:20140804
	FD_ZERO(&rd);
	FD_SET(fdevice, &rd);
	ret = select(fdevice + 1, &rd, NULL, NULL, &tv);
    if(ret > 0)
		lenth = read(fdevice, buf_char, size);
	else
		lenth = 0;
	(*env)->ReleaseByteArrayElements(env,buffer,buf_char,0);//M:Tim 20140804
	return lenth;
}

#define BUFFER_SIZE  128
//unsigned char buf[BUFFER_SIZE];

JNIEXPORT jint JNICALL JNI_ReadDev(JNIEnv *env,jobject obj, jbyteArray buffer, jint size,jint fdevice) {

//	fd2 = open("/dev/ttyHSL1", O_RDWR | O_NOCTTY);

	unsigned char *buf_char = (unsigned char*) ((*env)->GetByteArrayElements(env, buffer,NULL));
//	LOGD("---JNI---> JNI_ReadDev()  chuanru  size = %d", size);
	//M:begin
	int ret=0;
	int rc=0;
	int lenth = 0;
	int i,nlen = 0;

	int count=0;
	int index=0;

	fd_set rd;

	//jbyte *buf = (jbyte *)malloc(BUFFER_SIZE);
	unsigned char buf[BUFFER_SIZE];

	//struct timeval tv = { 0, 300*1000 };
	struct timeval tv = { 0, 50*1000 };

	memset(buf,0,BUFFER_SIZE);//M:20140804
	memset(buf_char,0,size);//M:20140804
	//while((buf_char - (unsigned char)buffer) < size) {//ensure data is not overflowed
	while(lenth < size) {
         FD_ZERO(&rd);
         FD_SET(fdevice, &rd);
         ret = select(fdevice + 1, &rd, NULL, NULL, &tv);
         if(ret > 0){
             if (FD_ISSET(fdevice,&rd)) {//has data
                 rc = read(fdevice, buf, BUFFER_SIZE);
                 if (rc > 0) {
                     memcpy(buf_char,buf,rc);
                     memset(buf,0,BUFFER_SIZE);//M-20150902
                     buf_char += rc;
					 nlen += rc;
                     lenth += rc;
                 }
             }
         }else if(ret == 0){
        	 //LOGE("---JNI---> JNI_ReadDev()  time out!\n");
             break;
         }else{
             //LOGE("---JNI---> JNI_ReadDev()  select error!\n");
             break;
         }
    }
	buf_char -= nlen;
	(*env)->ReleaseByteArrayElements(env,buffer,buf_char,0);//M:Tim 20140804
	return lenth;
}


static JNINativeMethod gMethods[] = {
		{ "jniOpenDev", "(Ljava/lang/String;IIII)I", (void*) JNI_OpenDev },
		{"jniCloseDev","(I)I",(void*)JNI_CloseDev },
//		{"init","(I)I",(void*)GPS_OnOff},
		{"jniRead","([BII)I",(void*)JNI_ReadDev},
		{"jniMinRead","([BII)I",(void*)JNI_MinReadDev},
		{"jniWrite","([BII)I",(void*)JNI_Write},
		{"laGaoOrLadi","(Ljava/lang/String;I)I",(void*)LaGaoOrLadi}

};

static int register_jni(JNIEnv *env) {
//	LOGD("--JNI  register_jni() \n");
	jclass clazz;
	int numMethods = 0;

	numMethods = sizeof(gMethods) / sizeof(gMethods[0]);
//	LOGD("register_fingerprint numMethods %d \n", numMethods);
	clazz = (*env)->FindClass(env, className);
	// LOGD("  s\n");
	if (clazz == NULL) {
	//	LOGE(" --JNI  clazz is null \n");
		return JNI_FALSE;
	}


	if ((*env)->RegisterNatives(env, clazz, gMethods, numMethods) < 0) {
	//	LOGE("--JNI   is fail \n");
		return JNI_FALSE;
	}

	return JNI_TRUE;
}


#ifdef __cplusplus
extern "C"
{
#endif

	jint JNI_OnLoad(JavaVM* vm, void* reserved) {
		jint result = JNI_ERR;
		JNIEnv* env = NULL;
		jclass clazz;
		int methodsLenght;
		if ((*vm)->GetEnv(vm, (void**) &env, JNI_VERSION_1_4) != JNI_OK) {
		//	LOGE("ERROR: GetEnv failed\n");
			return result;
		}
		assert(env != NULL);
		if (register_jni(env) <= 0) {
		//	LOGE("ERROR:  failed\n");
			return result;
		}
		//LOGD(" JNI_OnLoad e\n");
		result = JNI_VERSION_1_4;

		return result;
	}
#ifdef __cplusplus
}
#endif



